#include <iostream.h>
#include "util/PlatformUtils.hpp"
#include "PerlExceptionHandler.hpp"
#include "util/XMLString.hpp"
#include "util/XMLException.hpp"

// we initialize the exceptionHandler
SV *PerlExceptionHandler::exceptionHandler = NULL;

SV *
PerlExceptionHandler::setExceptionHandler(SV *codeRef) {
    SV *oldRef = &PL_sv_undef;	// default to 'undef'
    if (SvTYPE( SvRV(codeRef)) == SVt_PVCV) {
	if (exceptionHandler != NULL) {
	    oldRef = exceptionHandler;
	}
        SvREFCNT_inc(codeRef);
        exceptionHandler = codeRef;
    } else {
        cerr << "\nsetExceptionHandler:  "
	     << "invalid argument, must be code reference.\n" << endl;
    }
    return oldRef;
}

void 
PerlExceptionHandler::catchException(const XMLException &e) {
    if (!exceptionHandler) {
	cerr << "\nReceived Exception without handler\n"
	     << "Exception thrown in file: '" << e.getSrcFile() << "'\n"
	     << "At line: '" << e.getSrcLine() << "'\n"
	     << "Exception message is:  \n"
	     << XMLString::transcode(e.getMessage()) << "\n" << endl;
	XMLPlatformUtils::Terminate();
	die("Received Exception without handler");
    }
    dSP;

    ENTER;
    SAVETMPS;

    PUSHMARK(SP);

        // first argument is the element name
    char *class_name = "XML::Xerces::XMLException";
    HV *hash = newHV();
    HV *stash = gv_stashpv(class_name, FALSE);
    hv_magic(hash, 
    	 (GV *)sv_setref_pv(sv_newmortal(), 
    			    class_name, (void *)&e), 
    	 'P');
    XPUSHs(sv_bless(newRV_noinc((SV *)hash), stash));
    PUTBACK;

    perl_call_sv(exceptionHandler, G_VOID);

    FREETMPS;
    LEAVE;
}
